@@ -98,6 +98,9 @@ test_states <- function (sensor) {
         return()
     }
     expect_true(all(colnames(sensor$states) == c("tag", "start", "end", "value")))
+    expect_false(any(is.na(sensor$states$tag)))
+    expect_false(any(is.na(sensor$states$start)))
+    expect_false(any(is.na(sensor$states$end)))
     states <- dplyr::filter(sensor$states, .data$tag == myClim:::.model_const_SENSOR_STATE_SOURCE)
     if(length(sensor$values) == 0) {
         expect_equal(nrow(states), 0)
